#!/usr/bin/perl -w
#
# BATTS is the Barnhard Associates Trouble Ticket System.
# This is the CGI frontend.
#
# BATTS is copyright (C) 2001, 2002, 2003 
# Jon Lasser <jon@cluestickconsulting.com>
# Steve Killen <killen@xisp.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published
#   by the Free Software Foundation; either version 2 of the License,
#   or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#   02111-1307 USA
#

use strict;
use CGI qw/:standard/;
use Batts;

#########################
# subroutine declarations

sub do_params;
sub list_cust_tickets;
sub list_new_tickets;
sub list_user_tickets;
sub read_config;
sub single_ticket;
sub standard_footer;
sub standard_header;

#######################
# variable declarations

my $show_menu;

########################
# initialization stuff

$is_html="1"; 		# set HTML capability, as we're a Web page

read_config;		# local function to get info from the config file

batts_db_connect; 	# connect to the database and load table data

load_billing;
load_category;
load_customer;
load_people;
load_priorities;
load_status;


######################
# Main page

standard_header;

# check for any parameters first
$show_menu = do_params;

if ($show_menu) {
	list_new_tickets;
	list_cust_tickets;
	list_user_tickets;
	single_ticket;
}

standard_footer;

# end main page
#######################

################################################################
# do_params -- process parameters and do the appropriate output

sub do_params {

	# bookmarkable params...
	my $custname = param('customer_name');
	my $username = param('user_name');
	my $ticketid = param('ticketid');  # except when given with edit

	# utility params
	my $newtickets = param('new_tickets');
	my $edit = param('edit');
		my $custedit	= param('custedit');
		my $assedit		= param('assedit');
		my $statedit	= param('statedit');
		my $catedit		= param('catedit');
		my $priedit		= param('priedit');
		my $oldcust		= param('oldcust');
		my $oldass		= param('oldass');
		my $oldstat		= param('oldstat');
		my $oldcat		= param('oldcat');
		my $oldpri		= param('oldpri');
		
	my $shortname;

	# Customer ticket list
   if ($custname) {
      foreach (sort keys %customer) {
         if ($customer{$_}[1] eq $custname) {
            $shortname = $customer[$customer{$_}[0]];
         }
      }

      show_cust_ticket($shortname);
		return 0;
   }

	# Individual user list
	if ($username) {
      foreach (sort keys %people) {
         if ($people{$_}[1] eq $username) {
            $shortname = $people[$people{$_}[0]];
         }
      }

		show_my($shortname);
		return 0;
	}

	# Single ticket
	if (($ticketid) && (! $edit)) {
		show_ticket($ticketid);

		# show the log if asked
		if (param('log')) {
			show_ticket_log($ticketid);
		}

		return 0;
	}

	# New ticket flag -- hidden param
	if ($newtickets) {
		show_new;
		return 0;
	}

	# Edit ticket with changed values
	if ($edit) {
		if ($custedit ne $oldcust) {
			customer_ticket($ticketid,$custedit);
			print "Updated customer. ";
		}

		if ($assedit ne $oldass) {
			assign_ticket($ticketid,$assedit);
			print "Updated assignee. ";
		}

		# semi-broken status
		if (($statedit eq "pending on") ||
          ($statedit eq "resolved when") ||
          ($statedit eq "duplicate of")) {
			print "Status \"$statedit\" not implemented yet. ";
		} 
		elsif ($statedit ne $oldstat) {
			status_ticket($ticketid,$statedit);
			print "Updated status. "; 
		}

		if ($catedit ne $oldcat) {
			category_ticket($ticketid,$catedit);
			print "Updated category. ";
		}

		if ($priedit ne $oldpri) {
			priority_ticket($ticketid,$priedit);
			print "Updated priority. ";
		}

		# after updating, show ticket again
		show_ticket($ticketid);
      if (param('log')) {
         show_ticket_log($ticketid);
      }

		return 0;
	}

	# nothing to print, so show the menu
	return 1;  
}


#######################################################
# list_cust_tickets -- a form to show customer tickets
# params:	customer_name

sub list_cust_tickets {

	# build an array of full names for display
	my @fullnames;
	foreach (sort keys %customer) {
   	push @fullnames, $customer{$_}[1];
	}

	# make a form to select the customer tickets to display
	print	h3('Show a customer\'s tickets'),
		start_form,
		popup_menu(-name=>'customer_name',-value=>[@fullnames]),
		submit,
		end_form;
}

####################################################
# list_new_tickets -- a form to list all new tickets
# params:	new_tickets HIDDEN

sub list_new_tickets {

	print h3('Show all new tickets'),
		start_form,
		submit(-name=>'new_tickets',-value=>'Boom!'),
		end_form;
}

##############################################################
# list_user_tickets -- a form to show an individual's tickets
# params:   user_name

sub list_user_tickets {

   # build an array of full names for display
   my @fullnames;
   foreach (sort keys %people) {
      push @fullnames, $people{$_}[1];
   }

   # make a form to select the user's tickets to display
   print h3('Show an individual user\'s tickets'),
      start_form,
      popup_menu(-name=>'user_name',-value=>[@fullnames]),
      submit,
      end_form;
}


#######################################################################
# read_config -- initialize database variables (and others if needed)

sub read_config {
	my %config;

	open(CONFIG, "< $db_config_file") # get config file from a library variable
	or die "Couldn't open $db_config_file for reading: $!\n";

	while (<CONFIG>) {
		chomp;                  # no newline
		s/#.*//;                # no comments
		s/^\s+//;               # no leading white
		s/\s+$//;               # no trailing white
		next unless length;     # anything left?
		my ($var, $value) = split(/\s*=\s*/, $_, 2);
		$config{$var} = $value;
	}

	close(CONFIG);

	$host_name = $config{"host"};	# store values into library-global variables
	$user_name = $config{"user"};
	$password  = $config{"pass"};
}

############################################################
# single_ticket -- a form to show, well, a single ticket
# params: ticketid
sub single_ticket {
	
	print h3('Display a single ticket'),
			start_form,
			"Ticket ID ", textfield('ticketid'),
			submit, end_form;
}

#######################################
# standard_footer -- for completion

sub standard_footer {
	print end_html;
}

#################################
# standard_header -- always display this stuff first

sub standard_header { 
	my $href = "batts.cgi";
	print header,
			start_html('batts trouble ticket system'),
			"Welcome to ", a({href=>$href},'batts'),hr;
}
