# BATTS is the Barnhard Associates Trouble Ticket System.
# This is the Batts library, wherein springs all its might and fortitude.
#
# BATTS is copyright (C) 2001, 2002, 2003 
# Jon Lasser <jon@cluestickconsulting.com>
# Steve Killen <killen@xisp.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published
#   by the Free Software Foundation; either version 2 of the License,
#   or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#   02111-1307 USA
#

package Batts;
use strict;
use Getopt::Long;
use Mail::Internet;
use DBI;
use CGI qw/:standard/;

use vars qw(@ISA @EXPORT $VERSION);

use Exporter;
$VERSION = 0.1;              # Or higher
@ISA = qw(Exporter);

@EXPORT = qw(	$dbh          
					$dsn
					$is_html
					$db_config_file
					$default_config
					$host_name
					$user_name
					$password
					$return_address
					$sendmail
					$mail_resolution

					@ary         
					@status
					@people
					@billing
					@category
					@customer
					@priority

					%status     
					%people
					%billing
					%category
					%customer
					%priority
			
					assign_ticket
					billing_new
					category_new
					category_ticket
					close_ticket
					customer_new
					customer_ticket
					batts_db_connect
					duplicate_ticket
					html_event_footer
					html_event_header
					html_event_line
					html_summary_footer
					html_summary_header
					html_summary_line
					load_billing
					load_category
					load_customer
					load_people
					load_priorities
					load_status
					log_me
					log_end
					longdesc_ticket
					pending_on
					person_fullname
					person_new
					person_username
					priority_ticket
					requestor_ticket
					resolve_ticket
					resolve_when
					status_ticket
					shortdesc_ticket
					show_billing
					show_cat_ticket
					show_categories
					show_cust_ticket
					show_customers
					show_log
					show_my
					show_new
					show_open
					show_people
					show_priorities
					show_status
					show_ticket
					show_ticket_log
					text_event_footer
					text_event_header
					text_event_line
					ticket_new
					touch
			
                                   );       # Symbols to autoexport (:DEFAULT tag)

# @EXPORT_OK   = qw();       # Symbols to export on request
# %EXPORT_TAGS = (           # Define names for sets of symbols
#     TAG1 => [...],
#    TAG2 => [...],
#   ...
# );



########################
# variable declarations 
########################

our ($dbh);			# database standard stuff
our ($host_name, $user_name, $password) = (undef, undef, undef);
our (@ary);

our ($db_config_file) = "/etc/batts.myrc"; # Our database config file
our ($dsn) = "DBI:mysql:batts";            # DB DSN line, can be overridden
our ($default_config) = "/etc/battsrc";  # Our default configuration file

our ($return_address);		# Return address for mail
our ($sendmail);               	# Invocation of sendmail
our ($mail_resolution) = "1";   # default mail ticket resolution

our ($is_html) = "";            # no HTML output by default

our (@status, %status);		# Table of status codes, aka resolutions
our (@people, %people);		# Table of people --- see load_people
				# for documentation!
our (@priority, %priority);	# Table of priorities, to be read from
				# the database
our (@billing, %billing);	# Table of billing codes
our (@category, %category);	# Table of categories
our (@customer, %customer);	# Table of customers


########################
# format definitions   
########################

format SHORTDISPLAY_TOP =
Ticket  Created by                Assigned to  Status           Priority
------------------------------------------------------------------------
.


format SHORTDISPLAY =
@<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<< @<<<<<<<<<<@>>>>>>>>>>>>>
$ary[0],$ary[1],                  $ary[2],     $ary[4],   $ary[3]

Created on @<<<<<<<<<<<<<<<<<<<<  Last activity on @>>>>>>>>>>>>>>>>>>>> 
			  nicedate($ary[6]),								nicedate($ary[7])

		  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$ary[5]
------------------------------------------------------------------------
.

################################################################################
################################################################################
# function definitions 
################################################################################
################################################################################

#######################
# touch -- set the timestamp on a ticket
#######################

#######################
# assign_ticket -- set the responsible party on a ticket
#######################
 
sub assign_ticket($$) {

	my ($ticket_id) = $_[0];
	my ($assignee) = $_[1];

	my ($sth);

	my ($count) = 0;                        # counter variable

	if ($ticket_id < 1) {
		print STDERR
				"Ticket number must be a positive integer!\n";
		return -1;
	}

	if (! defined $people{$assignee}) {
		print STDERR 
				"No person \"$assignee\" is known\n";
		return -1;
	}

	return $dbh->do (
		qq{
			UPDATE tickets
			SET 	assignedto=$people{$assignee}[0],
					resolution=$status{'assigned'}
			WHERE ticket_id=$ticket_id
		}
	);
}

#######################
# billing_new -- add a new billing code to the database
########################

sub billing_new($) {

        my ($new_code) = $_[0];

        if (defined $billing{$new_code}) {
                print STDERR "Billing $new_code already in database\n";
                return -1;
        }

        return $dbh->do (qq{
                INSERT INTO billing_codes VALUES (NULL, "$new_code")
        });
}

#######################
#  category_new -- add a new category to the database
#######################
 
sub category_new($) {

        my ($new_category) = $_[0];

        if (defined $category{$new_category}) {
            print STDERR "Category $new_category already in database\n";
                return -1;
        }

        return $dbh->do (qq{
                INSERT INTO categories VALUES (NULL, "$new_category")
        });
}

#######################
# category_ticket -- set the category on a ticket
#######################

sub category_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($category_name) = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        if (! defined $category{$category_name}) {
                print STDERR "No category $category_name is known\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET category=$category{$category_name}
                WHERE ticket_id=$ticket_id
        });
}

#######################
# close_ticket -- close a ticket
#######################

sub close_ticket($) {

        my ($ticket_id) = $_[0];
        my ($sth);

        my ($count) = 0;                # loop counter variable
        my ($restext);                  # text of the matching tickets

	if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        # Find all matching tickets --- there can be only one
        $sth = $dbh->prepare (qq{
                SELECT restext
                FROM tickets
                WHERE ticket_id=$ticket_id
                AND resolution=$status{'resolved'}
        });

        $sth->execute();
        while (@ary = $sth->fetchrow_array()) {
                $count++;
                $restext=$ary[0];
        }
        $sth->finish();

        if ($count < 1) {
                print STDERR "Ticket $ticket_id is not resolved.\n";
                return -1;
        } elsif ($count > 1) {
                print STDERR "Multiple tickets $ticket_id.\n";
                return -1;
        }

        # OK, we only have one ticket. Change the status code in the
        # ticket now

	$dbh->do (qq{
                UPDATE tickets
                SET resolution=$status{'closed'}
                WHERE ticket_id=$ticket_id
        });

        # Now change tickets depending on this ticket
        update_tickets($ticket_id, $restext);
        return;
}

#######################
# customer_new -- add a new customer to the database
#######################

sub customer_new($$) {
        my ($customer_username) = $_[0];
        my ($customer_fullname) = $_[1];

        my ($quoted_username);
        my ($quoted_fullname);

        if (defined $customer{$customer_username}) {
		  print STDERR "Customer \"$customer_username\" already in database\n";
                  return -1;
        }

        $quoted_username=$dbh->quote($customer_username);
        $quoted_fullname=$dbh->quote($customer_fullname);
        return $dbh->do (qq{
                INSERT INTO customers VALUES
                        (NULL, $quoted_username, $quoted_fullname)
        });
}

#######################
# customer_ticket -- set the customer on a ticket
#######################

sub customer_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($customer_name) = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        if (! defined $customer{$customer_name}) {
                print STDERR "No customer \"$customer_name\" is known\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET customer=$customer{$customer_name}[0]
                WHERE ticket_id=$ticket_id
        });
}

#######################
# batts_db_connect -- connect to the batts database
#######################

sub batts_db_connect() {

# If given -p or --password, but no password given, grab it from
# # a real terminal:
        if (defined ($password) && !$password) {
                # turn off echoing but don't interfere with STDIN
                open (TTY, "/dev/tty") or die "Cannot open terminal\n";
                system("stty -echo < /dev/tty");
                print STDERR "Enter password: ";
                chomp ($password = <TTY>);
                system ("stty echo < /dev/tty");
                close (TTY);
                print STDERR "\n";
        }

        # Prepare the rest of the DSN
        $dsn .= ":hostname=$host_name" if $host_name;
        $dsn .= ";mysql_read_default_file=$db_config_file";

        # Connect to the database
        $dbh = DBI->connect($dsn, $user_name, $password,
                { RaiseError => 1 });
}

#######################
# duplicate_ticket -- mark a ticket as a duplicate of another ticket
#######################

sub duplicate_ticket($$) {

        my ($duplicate_ticket) = $_[0];
        my ($master_ticket) = $_[1];

        # Do we have a valid ticket number?
        if (($duplicate_ticket < 1) or ($master_ticket < 1)) {
                print STDERR
                       "Ticket number must be a positive integer!\n";
                return -1;
        }

        status_ticket($duplicate_ticket, "duplicate of");
        set_refticket($duplicate_ticket, $master_ticket);
}

#######################
# html_event_footer 
#######################

sub html_event_footer()
{
    print(qq{
      </TBODY>
    </TABLE>
  </BODY>
</HTML>
});
}

#######################
# html_event_header
#######################

sub html_event_header()
{
print(qq{

    <H1>Log of Events</H1>

    <TABLE BORDER="0" CELLSPACING="1" CELLPADDING="3">
      <THEAD>
        <TR>
          <TD WIDTH="10%"><STRONG>Event</STRONG></TD>
          <TD WIDTH="10%"><STRONG>Ticket</STRONG></TD>
          <TD WIDTH="17%"><STRONG>Logged By</STRONG></TD>
          <TD WIDTH="18%"><STRONG>Billing Code</STRONG></TD>
          <TD WIDTH="20%"><STRONG>Start Time</STRONG></TD>
          <TD WIDTH="20%"><STRONG>End Time</STRONG></TD>
          <TD WIDTH="5%"><STRONG>Private?</STRONG></TD>
        </TR>
        <TR>
          <TD WIDTH="100%" COLSPAN="5" <HR NOSHADE></TD>
        </TR>
      </THEAD>
      <TBODY>
});
}

#######################
# html_event_line
#######################

sub html_event_line(@)
{
print(qq{
        <TR>
          <TD>$_[0]</TD>
          <TD>$_[1]</TD>
          <TD>$_[2]</TD>
          <TD>$_[3]</TD>
          <TD>$_[5]</TD>
          <TD>$_[4]</TD>
          <TD>$_[7]</TD>
        </TR>
        <TR>
          <TD COLSPAN="5">$_[6]<HR NOSHADE></TD>
        </TR>
});
}

#######################
# html_summary_footer
#######################

sub html_summary_footer()
{
print(qq{
		</TBODY>
	</TABLE>
});
}

#######################
# html_summary_header
#######################

sub html_summary_header() {
print(qq{

    <H1>Summary of Tickets</H1>

    <TABLE BORDER="0" CELLSPACING="1" CELLPADDING="3">
      <THEAD>
        <TR>
          <TD WIDTH="10%"><STRONG>Ticket</STRONG></TD>
          <TD WIDTH="30%"><STRONG>Created by</STRONG></TD>
          <TD WIDTH="20%"><STRONG>Assigned to</STRONG></TD>
          <TD WIDTH="20%"><STRONG>Priority</STRONG></TD>
          <TD WIDTH="20%"><STRONG>Status</STRONG></TD>
        </TR>
        <TR>
          <TD WIDTH="100%" COLSPAN="5" <HR NOSHADE></TD>
        </TR>
      </THEAD>
      <TBODY>
});
}

#######################
# html_summary_line
#######################

sub html_summary_line(@) {
	my $href = "batts.cgi?ticketid=$_[0]";

	print "<TR>",
			"<TD>",a({href=>"$href"},$_[0]),"</TD>",
			"<TD>$_[1]</TD>",
			"<TD>$_[2]</TD>",
			"<TD>$_[3]</TD>",
			"<TD>$_[4]</TD>",
			"</TR>",
			"<TR>",
			"<TD>Created on:</TD>",
			"<TD>",nicedate($_[6]),"</TD>",
			"<TD></TD>",
			"<TD>Last activity on:</TD>",
			"<TD>",nicedate($_[7]),"</TD>",
			"</TR>",
			"<TR>",
			"<TD COLSPAN='5'>$_[5]<HR NOSHADE></TD>",
			"</TR>";
}

#######################
# log_me -- log an event associated with a ticket
#######################

sub log_me($$$$$) {

        my ($ticket_id) = $_[0];
        my ($username) = $_[1];
        my ($billcode) = $_[2];
        my ($is_private) = $_[3];
        my ($quoted_logmessage) = $_[4];

        my ($sth);

        my ($event_id);                 # New ID for this event
        my ($private);                  # Is this log entry private?

        # Do we have a valid ticket number?
        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        # Do we have a matching username?
        if (! defined $people{$username}) {
                print STDERR "No such person $username\n";
                return -1;
        }

        # Do we have a matching billing code?
        if (! defined $billing{$billcode}) {
                print STDERR "No such billing code $billcode\n";
                return -1;
        }

        # The private? field should be either Y or N. First, let's
        # uppercase it as just the first character
        $private = uc(substr($is_private, 0, 1));

        if (($private ne "Y") && ($private ne "N")) {
                print STDERR "<private?> must be Y or N\n";
                return -1;
        }

        # Create the !@#$% ticket already
        $dbh->do (qq{
                INSERT INTO events VALUES(
                        NULL,
                        $ticket_id,
                        $people{$username}[0],
                        $billing{$billcode},
                        NOW(),
                        NOW(),
                        $quoted_logmessage,
                        "$private")
        });

        # Now we need the ID of the generated event
        $sth = $dbh->prepare(qq{
                SELECT event_id
                FROM events
                WHERE ticket=$ticket_id
                  AND person=$people{$username}[0]
                  AND billable=$billing{$billcode}
                  AND logmessage=$quoted_logmessage
                  AND private="$private"
        });
        $sth->execute();
        $event_id = $sth->fetchrow_array();
        $sth->finish();

        $event_id= sprintf("%06d", $event_id);
        print STDERR ("New log entry created: $event_id\n");
        return $event_id;
}

#######################
# log_end -- set the end-time on a log event
#######################

sub log_end($) {

        my ($log_id) = $_[0];

        if ($log_id < 1) {
                print STDERR
                        "Log ID number must be a positive integer!\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE events
                SET endtime=NOW()
                WHERE event_id=$log_id
        });
}

#######################
# pending_on -- mark a ticket as pending on another ticket
#######################

sub pending_on($$) {

        my ($ticket_id) = $_[0];
        my ($master_ticket) = $_[1];

        # Do we have a valid ticket number?

        if (($ticket_id < 1) or ($master_ticket < 1)) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        status_ticket($ticket_id, "pending on");
        set_refticket($ticket_id, $master_ticket);
}

#######################
# person_fullname -- change the username of a person in the database
#######################

sub person_fullname($$) {

        my ($username) = $_[0];
        my ($fullname) = $_[1];

        my ($quoted_username);
        my ($quoted_fullname);

        if (! defined $people{$username}) {
                print STDERR "Person \"$username\" not in database\n";
                return -1;
        }

        $quoted_username = $dbh->quote($username);
        $quoted_fullname = $dbh->quote($fullname);

        return $dbh->do (qq{
                UPDATE people SET fullname=$quoted_fullname
                WHERE username=$quoted_username
        });
}

#######################
# person_new -- add a new person to the database
#######################

sub person_new($$$$$) {

	my ($person_username)=$_[0];
	my ($person_fullname)=$_[1];
	my ($person_email)=$_[2];
	my ($customer_name)=$_[3];
	my ($admin)=$_[4];

	my ($quoted_fullname);

	$person_username = lc($person_username);
	$person_email = lc($person_email);
	$quoted_fullname = $dbh->quote("$person_fullname");

	if (defined $people{$person_username}) {
		print STDERR "Person \"$person_username\" already in database\n";
		return -1;
	}

	if (! defined $customer{$customer_name}) {
		print STDERR "No such customer \"$customer_name\" known\n";
		return -1;
	}

	if (! (($admin eq "y") || ($admin eq "n"))) {
		print STDERR "Please specify admin status with a \"y\" or \"n\"."; 
		return -1;
	}

	my $isadmin = $admin eq "y" ? 1 : 0;

	$dbh->do ( qq {
		INSERT INTO people VALUES
		(NULL, "$person_username", $quoted_fullname, "$person_email", "$customer{$customer_name}[0]", $isadmin )
		});

	return 0;
}

#######################
# person_username -- change the username of a person in the database
#######################

sub person_username($$) {

        my ($email) = $_[0];
        my ($username) = $_[1];

        $username = lc($username);
        $email = lc($email);

        if (defined $people{$username}) {
              print STDERR "Person \"$username\" already in database\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE people SET username="$username"
                   WHERE email="$email"
        });
}

#######################
# priority_ticket -- change the priority of a ticket
#######################

sub priority_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($priority_name) = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        if (! defined $priority{$priority_name}) {
                print STDERR "No such priority $priority_name\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET priority=$priority{$priority_name}
                WHERE ticket_id=$ticket_id
        });
}

#######################
# requestor_ticket -- change the requesting user of a ticket
#######################

sub requestor_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($requestor) = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        if (! defined $people{$requestor}) {
                print STDERR "No such person $requestor\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET requestor=$people{$requestor}[0]
                WHERE ticket_id=$ticket_id
        });
}

#######################
# resolve_ticket -- resolve a ticket
#######################

sub resolve_ticket($$) {

        my ($mail);
        my ($ticket_id) = $_[0];
        my ($quoted_restext) = $_[1];

        my ($dequoted_restext);
        my ($person_id);
        my ($sth);
        my ($subject);

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        $dbh->do (qq{
                UPDATE tickets
                SET resolution=$status{'resolved'},
                    restext=$quoted_restext
                WHERE ticket_id=$ticket_id
        });

        # Update other tickets depending on this one
        update_tickets($ticket_id, $quoted_restext);

        # If the $mail_resolution option is not set, exit now
        return if (! $mail_resolution);

        # Now we need to get the information from the ticket.
        $sth = $dbh->prepare(qq{
                SELECT requestor,shortdesc, restext
                FROM tickets
                WHERE ticket_id=$ticket_id
        });
        $sth->execute();

        # this will only really execute once, so it's sort of silly
        while (@ary=$sth->fetchrow_array()) {
                $person_id=$ary[0];
                $subject=$ary[1];
                $dequoted_restext=$ary[2];
        }

        # Make the ticket ID pretty
        $ticket_id= sprintf("%06d", $ticket_id);

        # Fix up our subject
        $subject="Re: [$ticket_id] $subject";

        # And our body

        # Make up our response
        $mail= new Mail::Internet;
        $mail->body($dequoted_restext);
        $mail->head->replace("Subject", $subject);
        $mail->head->replace("From", $return_address);
        $mail->head->replace ("To", $people{$people[$person_id]}[1]);

        # Mail::Internet's send('sendmail') bit doesn't work, so we must
        open (SENDMAIL, "| $sendmail")
                or die ("Can't open sendmail!\n");
        $mail->print(\*SENDMAIL);
        close(SENDMAIL) or die ("I can't close sendmail!\n");

        return;
}

#######################
# resolve_when -- mark a ticket as resolved when another ticket is
#######################

sub resolve_when($$) {

        my ($ticket_id) = $_[0];
        my ($master_ticket) = $_[1];

        # Do we have a valid ticket number?
        if (($ticket_id < 1) or ($master_ticket < 1)) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        status_ticket($ticket_id, "resolved when");
        set_refticket($ticket_id, $master_ticket);
}

#######################
# show_billing -- print out a list of billing codes
#######################

sub show_billing() {
   my ($end_chars) = ($is_html) ? "<br>" : "\n";

	foreach (@billing) {
		if ($_) { print $_, $end_chars; }
	}
}

#######################
# show_cat_ticket -- display open tickets in a category
#######################

sub show_cat_ticket($) {

	my ($category_name) = $_[0];

	my ($category_id);              # ID number of category
	my ($query);                    # prepped query

	# Get category ID from lookup
	if (! defined $category{$category_name}) {
		print STDERR "Category $category_name does not exist\n";
		return -1;
	}
	$category_id = $category{$category_name};

	$query=qq {
		SELECT 
			tickets.ticket_id, 	#0
			creators.email,		#1
			assignees.username,	#2
			priorities.name, 		#3
			resolutions.name, 	#4
			tickets.shortdesc		#5
			tickets.creationtime	#6
			tickets.lastmodified	#7
		FROM 
			tickets, 
			priorities, 
			resolutions,
			people AS creators,
			people AS assignees
		WHERE 
			tickets.creator = creators.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id	
			AND tickets.resolution != $status{'closed'}
			AND tickets.resolution != $status{'resolved'}
			AND tickets.resolution != $status{'duplicate of'}
			AND tickets.category = $category_id
		ORDER BY 
			tickets.priority, 
			tickets.lastmodified,
			tickets.ticket_id
	};

	summary_print($query);

	return 0;
}

#######################
# show_categories -- print out a list of categories
#######################

sub show_categories() {
   my ($end_chars) = ($is_html) ? "<br>" : "\n";

	print join $end_chars, sort keys %category;
	print $end_chars;
}

#######################
# show_customers -- print out a list of customers
#######################

sub show_customers() {
   my ($end_chars) = ($is_html) ? "<br>" : "\n";

	foreach (sort keys %customer) {
		print "$_ : $customer{$_}[1]$end_chars";
	}
}

#######################
# show_cust_ticket --  display tickets created by a customer
#######################

sub show_cust_ticket($) {

	my ($custname) = $_[0];

	my ($custid);                   # ID number of customer
	my ($query);                    # prepped query

	# Get person ID from lookup
	if (! defined $customer{$custname}) {
		print STDERR "Customer ID $custname does not exist\n";
		return -1;
	}
	$custid = $customer{$custname}[0];

	$query=qq {
		SELECT 
			tickets.ticket_id,	#0
			creators.email,		#1
			assignees.username,	#2
			priorities.name, 		#3
			resolutions.name, 	#4
			tickets.shortdesc,	#5
         tickets.creationtime,#6
         tickets.lastmodified #7
		FROM 
			tickets, 
			priorities, 
			resolutions,
			people AS creators,
			people AS assignees
		WHERE 
			tickets.creator = creators.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id
			AND tickets.resolution != $status{'closed'}
			AND tickets.resolution != $status{'resolved'}
			AND tickets.resolution != $status{'duplicate of'}
			AND tickets.customer = $custid
		ORDER BY
			tickets.priority, 
			tickets.lastmodified,
			tickets.ticket_id
	};

	summary_print($query);

	return 0;
}

#######################
# show_log -- print out a single log entry
#######################

sub show_log($) {

	my ($event_id) = $_[0];

	# If our log request is for an event < 1, get rid of it
	if ($event_id < 1) {
		print STDERR 
				"Requested log entry < 1!\n";
		return -1;
	}

	my ($query) = qq {
		SELECT 
			event_id, 				#0
			ticket, 					#1
			people.username,		#2
			billing_codes.name, 	#3
			endtime, 				#4
			starttime, 				#5
			logmessage,				#6
			private					#7
		FROM 
			events, 
			people, 
			billing_codes
		WHERE 
			events.person = people.person_id
			AND events.billable = billing_codes.billable_id
			AND event_id="$event_id"
	};

	log_print($query);

	return 0;
}

#######################
# show_my --  display tickets created by or belonging to a user
#######################

sub show_my($) {

	my ($username) = $_[0];

	my ($person);                   # ID number of individual
	my ($query);                    # query to pass off

	# If person not in tables, get rid of it.
	if (! defined $people{$username}) {
		print STDERR "No person $username in tables\n";
		return -1
	}

	$person = $people{$username}[0];

	# Set our output formats
	$query=qq {
		SELECT 
			tickets.ticket_id,	#0 
			creators.email,		#1
			assignees.username,	#2
			priorities.name, 		#3
			resolutions.name, 	#4
			tickets.shortdesc,	#5
         tickets.creationtime,#6
         tickets.lastmodified #7
		FROM 
			tickets, 
			priorities, 
			resolutions,
			people AS creators,
			people AS assignees
		WHERE 
			tickets.creator = creators.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id
			AND tickets.resolution != $status{'closed'}
			AND tickets.resolution != $status{'resolved'}
			AND tickets.resolution != $status{'duplicate of'}
			AND ((tickets.creator LIKE "$person")
				OR (tickets.assignedto LIKE "$person"))
		ORDER BY 
			tickets.priority, 
			tickets.lastmodified,
			tickets.ticket_id
	};

	summary_print($query);

	return 0;
}

#######################
# show_new -- display short form for all new tickets
#######################

sub show_new() {

	my ($query);            # Our query to send out

	$query = qq {
		SELECT 
			tickets.ticket_id,	#0 
			creators.email,		#1
			assignees.username,	#2
			priorities.name, 		#3
			resolutions.name, 	#4
			tickets.shortdesc,	#5
         tickets.creationtime,#6
         tickets.lastmodified #7
		FROM 
			tickets, 
			priorities, 
			resolutions,
			people AS creators,
			people AS assignees
		WHERE 
			tickets.creator = creators.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id
			AND tickets.resolution = $status{'new'}
		ORDER BY 
			tickets.creationtime, 
			tickets.lastmodified,
			tickets.ticket_id
	};

	summary_print($query);

	return 0;
}

#######################
# show_open -- display short form for all open tickets
#######################

sub show_open() {

	my ($query);                    # our query to run

	$query=qq {
		SELECT 
			tickets.ticket_id,	#0 
			creators.email,		#1
			assignees.username,	#2
			priorities.name, 		#3
			resolutions.name, 	#4
			tickets.shortdesc,	#5
         tickets.creationtime,#6
         tickets.lastmodified #7
		FROM 
			tickets, 
			priorities, 
			resolutions,
			people AS creators,
			people AS assignees
		WHERE 
			tickets.creator = creators.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id
			AND tickets.resolution != $status{'closed'}
			AND tickets.resolution != $status{'resolved'}
			AND tickets.resolution != $status{'duplicate of'}
		ORDER BY 
			tickets.priority, 
			tickets.lastmodified,
			tickets.ticket_id
	};

	summary_print($query);

	return 0;
}

#######################
# show_people -- print out a list of people
#######################

sub show_people() {
   my ($end_chars) = ($is_html) ? "<br>" : "\n";

	foreach (sort keys %people) {
		print "$_ : $people{$_}[1] : $people{$_}[3] : $customer[$people{$_}[2]] $end_chars";
	}

}

#######################
# show_priorities -- print out a list of priorities
#######################

sub show_priorities() {
	my ($end_chars) = ($is_html) ? "<br>" : "\n";

	foreach (@priority) {
		if ($_) { print "$_ $end_chars"; }  # the if filters out blank array elements
	}
}

#######################
# show_status -- print out a list of status codes
#######################

sub show_status() {
   my ($end_chars) = ($is_html) ? "<br>" : "\n";

	foreach (@status) {
		if ($_) { print "$_ $end_chars"; }
	}
}

#######################
# show_ticket -- print out a ticket (long-form)
#######################

sub show_ticket($) {

	my ($ticket_id) = $_[0];

	if ($ticket_id < 1) {
		print STDERR
				"Ticket number must be a positive integer!\n";
		return -1;
	}

	# the query
	my ($query) = qq {
		SELECT 
			creators.username, 		#0
			creators.email, 			#1
			customers.username,		#2
			assignees.username, 		#3
			assignees.email, 			#4
			categories.name,			#5
			priorities.name, 			#6
			tickets.creationtime,	#7
			tickets.lastmodified,	#8
			tickets.shortdesc, 		#9
			tickets.longdesc, 		#10
			tickets.refticket,		#11
			resolutions.name, 		#12
			tickets.restext, 			#13
			requestor.username		#14
		FROM 	
			tickets, 
			customers, 
			categories, 
			priorities, 
			resolutions,
			people AS creators,
			people AS requestor,
			people AS assignees
		WHERE 
			tickets.customer = customers.customer_id
			AND tickets.creator = creators.person_id
			AND tickets.requestor = requestor.person_id
			AND tickets.assignedto = assignees.person_id
			AND tickets.category = categories.category_id
			AND tickets.priority = priorities.priority_id
			AND tickets.resolution = resolutions.resolution_id
			AND tickets.ticket_id = $ticket_id
	}; # end query

	# print the html or text ticket
	ticket_print($ticket_id, $query);

	return 0;
}

#######################
# show_ticket_log -- print out the log(s) for a ticket
#######################

sub show_ticket_log($) {

	my ($ticket_id) = $_[0];

	# If our log request is for a ticket < 1, get rid of it
	if ($ticket_id < 1) {
		print STDERR 
				"Requested ticket number < 1!\n";
		return -1;
	}

	my ($query) = qq {
		SELECT 
			event_id,				#0 
			ticket, 					#1
			people.username,		#2
			billing_codes.name, 	#3
			endtime, 				#4
			starttime, 				#5
			logmessage,				#6
			private					#7
		FROM 
			events, 
			people, 
			billing_codes
		WHERE 
			events.person = people.person_id
			AND events.billable = billing_codes.billable_id
			AND events.ticket="$ticket_id"
	};

	log_print($query);

	return 0;
}

#######################
# status_ticket --  set the status of a ticket to the given priority
#######################

sub status_ticket($$) {

        my $ticket_id = $_[0];
        my $statusname = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }


        if (! defined $status{$statusname}) {
		print STDERR "Status $statusname does not exist in our tables.\n";
        	return -1;
        }

        return $dbh->do (qq{
	        UPDATE tickets
        	SET resolution=$status{$statusname}
	        WHERE ticket_id=$ticket_id
        });

}

#######################
# text_event_footer
#######################

sub text_event_footer() {
        # We don't do anything here now
}

#######################
# text_event_header
#######################

sub text_event_header() {

        print "Event  Ticket Logged By    Billing Code Start Time     ";
        print "End Time  Private\n";
        print "------------------------------------";
        print "------------------------------------\n";

}

#######################
# text_event_line
#######################

sub text_event_line(@) {

        printf "%06d %06d %-12s %-12s %-14s %-14s %1s\n",
           $_[0], $_[1], $_[2], $_[3], $_[5], $_[4], $_[7];
        print "$_[6]\n";
        print "------------------------------------";
        print "------------------------------------\n";
}

#######################
# ticket_new -- create a new ticket
#######################

sub ticket_new($$$) {

	my ($email) = $_[0];
	my ($shortdesc) = $_[1];
	my ($quoted_longdesc) = $_[2];

	my ($sth);

	my ($loop);                     # travel through the hash
	my ($found) = 0;                # Have we found our answer yet?
	my ($answer) = "";              # What is that answer?
	my ($quoted_shortdesc);         # safe for the database
	my ($ticket_id);                # nice copy of new ticket number

	# All of our numerous sanity checks
	if (! defined $category{'unsorted'}) {
		print STDERR "No category \"unsorted\" in database!\n";
		return -1;
	}

	if (! defined $people{'unassigned'}) {
		print STDERR "No person \"unassigned\" in database!\n";
		return -1;
	}

	if (! defined $priority{'medium'}) {
		print STDERR "No priority \"medium\" in database!\n";
		return -1;
	}

	if (! defined $status{'new'}) {
		print STDERR "No status \"new\" in database!\n";	
		return -1;
	}

	if (! defined $customer{'unknown'}) {
		print STDERR "No customer \"unknown\" in database!\n";
		return -1;
	}

	# OK, now that we're sane, find the matching customer e-mail
	foreach $loop (keys %people) {
		if ($people{$loop}[1] =~ /$email/) {
			$found=1;
			$answer=$loop;
		}
	}

	# if not found, we'll create a new person and try again
	if (! $found) {
		person_new("tempname", "New Person", $email, "unknown", "n");
		load_people();
		print STDERR "User ID 'tempname' created for $email\n";
		print STDERR "You should probably change that.\n";

		#ampersand suppresses a warning about calling too early
		return &ticket_new($email, $shortdesc, $quoted_longdesc);
	}

	# Protect quotes in the short description
	$quoted_shortdesc=$dbh->quote($shortdesc);

	# Create the !@#$% ticket already
	$dbh->do (qq{
		INSERT INTO tickets VALUES(
			NULL,
			$people{$answer}[0],
			$people{$answer}[0],
			$people{$answer}[2],
			$category{'unsorted'},
			$priority{'medium'},
			NULL,
			NULL,
			$people{'unassigned'}[0],
			$quoted_shortdesc,
			$quoted_longdesc,
			NULL,
			$status{'new'},
			NULL)
		});

	# Now we need the ID of the new ticket
	$sth = $dbh->prepare(qq{
		SELECT ticket_id
		FROM tickets
		WHERE resolution=$status{'new'}
		AND assignedto=$people{'unassigned'}[0]
		AND longdesc=$quoted_longdesc
	});
	$sth->execute();
	$ticket_id = $sth->fetchrow_array();
	$sth->finish();

	$ticket_id = sprintf("%06d", $ticket_id);
	print STDERR ("New ticket created: $ticket_id\n");

	return $ticket_id;
}

#######################
# load_billing
#######################

sub load_billing() {

	my ($sth);

	# Get rid of the warning
	@billing="";

	$sth = $dbh->prepare (qq{
		SELECT billable_id, name
		FROM billing_codes
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$billing[$ary[0]] = $ary[1];
		$billing{$ary[1]} = $ary[0];
	}

	# Clean up
	$sth->finish;
}

#######################
# load_category
#######################

sub load_category() {

	my ($sth);

	# Get rid of the warning
	@category="";

	$sth = $dbh->prepare (qq{
		SELECT 
			category_id, 
			name
		FROM 
			categories
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$category[$ary[0]] = $ary[1];	# $category[id] = name
		$category{$ary[1]} = $ary[0];	# $category{name} = id
	}

	# Clean up
	$sth->finish;
}

#######################
# load_customer
#######################

sub load_customer() {

	my ($sth);

	# Get rid of the warning
	@customer="";

	$sth = $dbh->prepare (qq{
		SELECT 
			customer_id, 
			username, 
			fullname
		FROM 
			customers
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$customer[$ary[0]] = $ary[1];		# $customer[cust_id] = username
		$customer{$ary[1]}[0] = $ary[0];	# $customer{username}[0] = cust_id
		$customer{$ary[1]}[1] = $ary[2];	# $customer{username}[1] = fullname
	}

	# Clean up
	$sth->finish;
}

#######################
# load_people
#######################
# The people array just returns the name based on the ID number,
# but the hash (with name as key) returns (id, e-mail, customer-id)

sub load_people() {

	my ($sth);

	# Get rid of the warning
	@people="";

	$sth = $dbh->prepare (qq{
		SELECT 
			person_id, 
			username, 
			email, 
			customer, 
			fullname, 
			isadmin
		FROM 
			people
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$people[$ary[0]] = $ary[1];		# people[id] = username
		$people{$ary[1]}[0] = $ary[0];	# people{username}[0] = id
		$people{$ary[1]}[1] = $ary[2];	# people{username}[1] = email
		$people{$ary[1]}[2] = $ary[3];	# people{username}[2] = customer
		$people{$ary[1]}[3] = $ary[4];	# people{username}[3] = fullname
		$people{$ary[1]}[4] = $ary[5];	# people{username}[4] = isadmin
	}

	# Clean up
	$sth->finish;
}

#######################
# load_priorities
#######################

sub load_priorities() {

	my ($sth);

	# Get rid of the warning
	@priority="";

	$sth = $dbh->prepare (qq{
		SELECT priority_id, name
		FROM priorities
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$priority[$ary[0]] = $ary[1];
		$priority{$ary[1]} = $ary[0];
	}

	# Clean up
	$sth->finish;
}

#######################
# load_status
#######################

sub load_status() {

	my ($sth);

	# Get rid of the warning
	@status="";

	$sth = $dbh->prepare (qq{
		SELECT 
			resolution_id, 
			name
		FROM 
			resolutions
	});
	$sth->execute;

	# We load both a hash and an array, so we can lookup either direction
	while (@ary = $sth->fetchrow_array) {
		$status[$ary[0]] = $ary[1];	# $status[id] = name
		$status{$ary[1]} = $ary[0];	# $statud{name} = id
	}

	# Clean up
	$sth->finish;
}

#######################
# longdesc_ticket -- set the long description of a ticket
#######################

sub longdesc_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($quoted_longdesc) = $_[1];

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET longdesc=$quoted_longdesc
                WHERE ticket_id=$ticket_id
        });

}

#######################
# shortdesc_ticket set the short description of a ticket
#######################

sub shortdesc_ticket($$) {

        my ($ticket_id) = $_[0];
        my ($shortdesc) = $_[1];

        my ($quoted_shortdesc);

        if ($ticket_id < 1) {
                print STDERR
                        "Ticket number must be a positive integer!\n";
                return -1;
        }

        # Protect quotes in the short description
        $quoted_shortdesc=$dbh->quote("$shortdesc");

        return $dbh->do (qq{
                UPDATE tickets
                SET shortdesc=$quoted_shortdesc
                WHERE ticket_id=$ticket_id
        });

}

###################################################################################
# Non-public functions
#    here there be dragons.
############

#######################
# clear_refticket -- erase the refticket for the given ticket
#######################

sub clear_refticket($) {

       my ($ticket_id) = $_[0];

       if ($ticket_id < 1) {
               print STDERR
                       "Ticket number must be a positive integer!\n";
               return -1;
       }

       return $dbh->do (qq{
               UPDATE tickets
               SET refticket=NULL
               WHERE ticket_id=$ticket_id
       });
}

#######################
# log_print -- display event log based on query passed     
#######################

sub log_print($) {

	my ($query) = @_;

	my ($sth);

	# Set our output formats
	if ($is_html) {
		html_event_header;
	} else {
		text_event_header;
	}

	$sth = $dbh->prepare($query);
	$sth->execute();

	# print all matching events
	while (@ary = $sth->fetchrow_array()) {

		($ary[4],$ary[5]) = nicedate($ary[4],$ary[5]);
					
		if ($is_html) {
			html_event_line(@ary);
		} else {
			text_event_line(@ary);
		}
	}

	# Clean up
	$sth->finish();
	
	if ($is_html) {
		html_event_footer;
	} else {
		text_event_footer;
	}

	return 0;
}

####################
# nicedate -- convert database dates into human-readable form for printing
####################

sub nicedate {
	my ($yyyy,$mm,$dd,$hh,$mimi,$ss);
	my %months = ( "01"=>"Jan", "02"=>"Feb", "03"=>"Mar", "04"=>"Apr",
						"05"=>"May", "06"=>"Jun", "07"=>"Jul", "08"=>"Aug",
						"09"=>"Sep", "10"=>"Oct", "11"=>"Nov", "12"=>"Dec" );

	foreach (@_) {
 		($yyyy,$mm,$dd,$hh,$mimi,$ss) = unpack("A4 A2 A2 A2 A2 A2",$_);
		$_ = "$dd $months{$mm} $yyyy, $hh:$mimi:$ss";
	}

	return @_;
}


#######################
# set_refticket -- set the refticket for the given ticket
#######################

sub set_refticket($$) {

        my ($ticket_id) = $_[0];
        my ($refticket) = $_[1];

        if (($ticket_id < 1) or ($refticket < 1)) {
                print STDERR "Ticket number must be a positive integer!\n";
                return -1;
        }

        return $dbh->do (qq{
                UPDATE tickets
                SET refticket=$refticket
                WHERE ticket_id=$ticket_id
        });
}

#######################
# summary_print -- display short-form tickets based on query passed
#######################

sub summary_print($) {

        my ($query) = @_;
        my ($sth);

        # Set our output formats
        if ($is_html) {
                html_summary_header;
        } else {
                $~ = "SHORTDISPLAY";
                $^ = "SHORTDISPLAY_TOP";
        }

        $sth = $dbh->prepare($query);
        $sth->execute();

        # print all matching tickets
        while (@ary = $sth->fetchrow_array()) {
                if ($is_html) {
                        html_summary_line(@ary);
                } else {
                        write;
                }
        }

        # Clean up
        $sth->finish();
        if ($is_html) {
                html_summary_footer;
        } else {
                $~ = "STDOUT";
                $^ = "STDOUT_TOP";
        }

        return 0;
}

#######################
# ticket_print -- display individual tickets based on query passed
# params:	edit	SUBMIT
#		ticketid,	custedit,	assedit,	statedit,	catedit,	priedit
#						oldcust,		oldass,	oldstat,		oldcat,	oldpri
#######################

sub ticket_print ($$) {

	my ($ticket_id) = $_[0];
	my ($query) = $_[1];
	my ($sth);
	my ($href);

	$sth = $dbh->prepare($query);
	$sth->execute();

	# print the ticket
	while (@ary = $sth->fetchrow_array()) {

	   # Make dates human-readable for both text and html
	   nicedate($ary[7],$ary[8]);

		# begin text ticket

	   if (! $is_html) {  

			print ("Ticket Number $ticket_id:\n");
			print ("Created by $ary[0] <$ary[1]> on $ary[7]\n");

			if (($ary[14] ne 'unassigned') && ($ary[14] ne $ary[0])) {
				print ("Current ticket requestor is $ary[14] <$people{$ary[14]}[1]>\n");
			}

			print ("Ticket customer is $ary[2]\n");
			print ("Assigned to $ary[3] <$ary[4]>; ");
			print ("last modified on $ary[8]\n");
			print ("Status is $ary[12] ");
                	
			if (defined $ary[11]) { 
				print ("$ary[11]\n"); 
			} else { 
				print ("\n"); 
			}

			print ("Category: $ary[5]\t\t\tPriority: $ary[6]\n\n");
			print ("$ary[9]\n\n$ary[10]\n");
                
			if (defined $ary[13]) {
				print ("Resolution: $ary[13]\n");
			}							

	# end text ticket

	} else {						

	# begin html ticket

			print start_form;
			
			print "Ticket #$ticket_id:",br,
					b("Created by: "),$ary[0]," &lt;$ary[1]&gt; ",br,
						hidden('ticketid',"$ticket_id");

			# print current requestor too
			if (($ary[14] ne 'unassigned') && ($ary[14] ne $ary[0])) {
				print b("Current requestor: "),
						$ary[14]," &lt;$people{$ary[14]}[1]&gt;",
						br;
			}

			# eliminate spaces for our selection fields
			my @temp_cust = grep { $_ } @customer;
			my @temp_stat = grep { $_ } @status;
			my @temp_cat = grep { $_ } @category;
			my @temp_pri = grep { $_ } @priority;

			# only list admins to assign a ticket
			my @temp_ass = grep { $_ && $people{$_}[4] } @people;

			# labels for assignees (username/email)
			my %ass_labels;
			foreach (@temp_ass) {
				$ass_labels{$_} = "$people{$_}[3] \<$people{$_}[1]\>";
			}

			print b("Ticket customer: "),
						popup_menu('custedit',\@temp_cust,$ary[2]),br,
						hidden('oldcust',$ary[2]),
					b("Assigned to: "),
						popup_menu('assedit',\@temp_ass,$ary[3],\%ass_labels),br,
						hidden('oldass',$ary[3]),
					b("Created on: "),$ary[7],br,
					b("Last modified: "),$ary[8],br,
					b("Status: "), 
						popup_menu('statedit', \@temp_stat, $ary[12]),br,
						hidden('oldstat',$ary[12]);

			# duplicate ticket notice if applicable
			if (defined $ary[11]) {
				$href = "batts.cgi?ticketid=$ary[11]";
				print "This ticket is a duplicate of ",a({href=>"$href"},$ary[11]),br;
			} else {
				print br;
			}

			print "Category: ",
						popup_menu('catedit',\@temp_cat,$ary[5]),br,
						hidden('oldcat',$ary[5]),
					"Priority: ",
						popup_menu('priedit',\@temp_pri,$ary[6]),p,
						hidden('oldpri',$ary[6]);
				
			# our buttons
			print submit(-name=>"edit", -value=>"Update Ticket"),	
					reset("Reset Values"), p;

         # link to the log unless it's being shown
         unless (param('log')) {
            $href = "batts.cgi?ticketid=$ticket_id&log=1";
            print a({href=>$href},"Show log"),p;
         }

			# The ticket body			
			print "$ary[9]",p,
					"$ary[10]",br;

			if (defined $ary[13]) {
				print "Resolution: $ary[13]",br;
			}
		}							

	# end html ticket

	}

	# Clean up
	$sth->finish();

	return 0;
}

#######################
# touch -- set the timestamp on a ticket
#######################

sub touch($) {

	my ($ticket_id) = $_[0];

	if ($ticket_id < 1) {
		print STDERR "Ticket number must be a positive integer!\n";
		return -1;
	}

	return $dbh->do (qq{
		UPDATE 
			tickets
		SET 
			lastmodified=NOW()
		WHERE 
			ticket_id=$ticket_id
	});
}

#######################
# update_tickets --  update tickets based on the incoming ticket info
#######################

sub update_tickets($$) {

        my ($sth);
        my ($refticket) = $_[0];
        my ($restext) = $_[1];

        my ($logmessage);

        # resolve tickets that are resolved when or duplicate of this
        $logmessage = $dbh->quote("Resolved when $refticket resolved.");

        $sth = $dbh->prepare (qq{
                SELECT ticket_id
                FROM tickets
                WHERE refticket=$refticket
                AND ((resolution=$status{'resolved when'})
                  OR (resolution=$status{'duplicate of'}))
        });
        $sth->execute();

        while (@ary = $sth->fetchrow_array()) {
                log_me($ary[0], 'unassigned', 'informational', "N",
                    $logmessage);
                clear_refticket($ary[0]);
                resolve_ticket($ary[0],$restext);
        }
        $sth->finish();

        # set assigned all tickets pending on this ticket
    $logmessage = $dbh->quote("Status reset when $refticket resolved.");
        $sth = $dbh->prepare (qq{
                SELECT ticket_id, assignedto
                FROM tickets
                WHERE refticket=$refticket
                AND resolution=$status{'pending on'}
        });
        $sth->execute();

        while (@ary = $sth->fetchrow_array()) {
                clear_refticket($ary[0]);
                log_me($ary[0], 'unassigned', 'informational', "N",
                        $logmessage);

                if ($ary[1] = $people{'unassigned'}) {
                        status_ticket($ary[0], 'new');
                } else {
                        status_ticket($ary[0], 'assigned');
                }
        }

        $sth->finish();
        return;
}


1;                           # this should be your last line
 
